# -*- coding: utf-8 -*-

import const
import json
import metzoo
import requests
import time
import calendar

from .comparable import Comparable
from .errors import AgentCreationError, AgentDataError, InvalidData
from .request_info import RequestInfo

class Agent(Comparable):
    """Interface to communicate with metzoo services

    To create an agent just pass a name and your customer key:

        >>> agent = metzoo.Agent('xxx-xxx....', 'name')
        <Agent 'name', key: 'xxx-xxx...'>

    An agent is used to send data to metzoo:

        >>> agent.send({"single.metric": 123})
        {"status":"ok"}

        >>> agent.send({"multi.metric1": 123, "multi.metric2": 456})
        {"status":"ok"}

    And can update it's own configuration:

        >>> agent.update()
    """

    REQ_INFO = \
            { "send": RequestInfo("POST", "/metrics/data?register=true")
            , "create": RequestInfo("POST", "/agents")
            , "set_unit": RequestInfo("PUT", "/metrics")
            }

    HEADERS = \
            { "Content-Type": "application/json"
            , "User-Agent": "metzoo-python-sdk/" + metzoo.__version__
            }

    AGENT_KEYS = {}

    def __init__(self, customer_key, name, url="http://api.metzoo.com"):
        """Initializes an agent for the given customer_key.

        If the agent doesn't exists it creates one.
        If it exists it load the configuration from metzoo servers.

        You can set the url to use your own api
        """

        assert customer_key is not None, "You must provide a customer key when creating an Agent"
        assert name is not None, "You must provide a name when creating an Agent"

        self.__customer_key = customer_key
        self.__name = name
        self.__url = url
        self.__key = self.__get_agent_key(customer_key, name)

    @property
    def name(self):
        """Returns the name of the agent"""
        return self.__name

    @property
    def url(self):
        """Returns the url of the api that the agent is communicating with"""
        return self.__url

    @property
    def customer_key(self):
        """Returns the customer key of the agent"""
        return self.__customer_key

    @property
    def key(self):
        """Returns the key of the agent"""
        return self.__key

    def configure(self, config):
        """It configures the agent, sets the unit for every metric

        config is a dict with metric names as keys and metric unit as value

        returns the response (as a json dict) of the request sent

        Example:
            >>> agent.configure({"metric.name": "Kilobytes", "metric.name2": "°C"})
            {"status":"ok"}
        """

        if type(config) is not dict:
            raise InvalidData(got=type(config), expected=dict)

        if len(config) == 0:
            raise InvalidData(got="an empty dict", expected="a dict with at least one key:value pair")

        units = []
        for metric in config:
            unit = config[metric]
            if type(unit) is not str:
                raise InvalidData(got=type(unit), expected="{} for the unit of {}".format(str, metric))
            units.append({"name": metric, "unit": unit})

        return self.__get_valid_response("set_unit", units, {"Agent-Key": self.key})

    def send(self, metrics):
        """For every key-value pair in the dictionary metrics it will create the metric if it doesn't exists and it will
        send the data for that metric.

        metrics is a dict with metric names as keys and metric data as values

        returns the response (as a json dict) of the request sent

        Example:
            >>> agent.send({"metric.name": 123})
            {"status":"ok"}
        """

        if type(metrics) is not dict:
            raise InvalidData(got=type(metrics), expected=dict)

        if len(metrics) == 0:
            raise InvalidData(got="an empty dict", expected="a dict with at least one key:value pair")

        data = {"t": calendar.timegm(time.gmtime())}
        for metric in metrics:
            value = metrics[metric]
            if not type(value) in [int, float, long]:
                raise InvalidData(
                        got=type(value),
                        expected="{}, {} or {} for the value of {}".format(int, float, long, metric))
            data[metric] = value

        return self.__get_valid_response("send", data, {"Agent-Key": self.key})

    def update(self):
        """It downloads the configuration from metzoo server and update itself with it.
        """
        pass

    def __repr__(self):
        return "<Agent '%s', key: %s>" % (self.name, self.key)

    def __get_agent_key(self, customer_key, name):
        key = None
        try:
            key = self.AGENT_KEYS[name]
        except KeyError:
            key = self.AGENT_KEYS[name] = self.__create_agent_key(customer_key, name)
        return key

    def __create_agent_key(self, customer_key, name):
        status, response = self.__request("create", {"host_name": name, "customer_key": customer_key})
        if not status in [200, 201]:
            raise AgentCreationError(status, response)
        return response

    def __request(self, info_name, body=None, headers={}):
        info = self.REQ_INFO[info_name]
        headers.update(self.HEADERS)
        print "\n--> Request:\n  method: {}\n  url: {}\n  body: {}\n  headers: {}".format(
                info.method, self.url + info.route, json.dumps(body), headers)
        response = requests.request(info.method, self.url + info.route, data=json.dumps(body), headers=headers)
        print "<-- Response:\n  status: {}\n  text: {}".format(response.status_code, json.loads(response.text))
        return (response.status_code, json.loads(response.text))

    def __get_valid_response(self, info_name, body, headers):
        status, response = self.__request(info_name, body, headers)
        if not status in range(200, 300):
            raise AgentDataError(status, response)
        return response
